from leapp.actors import Actor
from leapp.models import InstalledRedHatSignedRPM, RpmTransactionTasks
from leapp.libraries.stdlib import api, CalledProcessError, run
from leapp.tags import PostProcessingScriptPhaseTag, IPUWorkflowTag

class RecoverBCCRpm(Actor):
    """
    The actor will install the bcc rpm.
    """
    name = 'bcc_rpm_recover'
    consumes = (RpmTransactionTasks, )
    produces = ()
    tags = (PostProcessingScriptPhaseTag, IPUWorkflowTag)

    def process(self):
        bcc_need_recover = False

        for tasks in api.consume(RpmTransactionTasks):
            for rm_pkg_name in tasks.to_remove:
                if 'bcc' in rm_pkg_name:
                    bcc_need_recover = True
                    break

        if bcc_need_recover:
            os_version = '8.2'
            with open('/etc/os-release') as f:
                data = dict(l.strip().split('=', 1) for l in f.readlines() if '=' in l)
                os_version = data.get('VERSION', '').strip('"')

            if '8.2' in os_version:
                module_name = 'bcc-dep:el8.4'
                os_version = '8.2'
            elif '8.4' in os_version:
                module_name = 'bcc-dep:an8.5'
                os_version = '8.4'
            elif '8.6' in os_version:
                module_name = 'TO:DO'
                os_version = '8.6'
            else:
                api.current_logger().warning("Failed to get target OS version: {}".format(os_version))

            with open("/etc/yum.repos.d/BCLinux-Plus.repo", 'w') as fd:
                fd.write('''
[Plus]
name=BC-Linux-{0} - Plus
baseurl=http://mirrors.bclinux.org/bclinux/el{1}/Plus/$basearch/os/
gpgcheck=0
enabled=1
gpgkey=file:///etc/pki/rpm-gpg/RPM-GPG-KEY-BC-Linux-8
                '''.format(os_version, os_version))

            run(['yum', 'clean',  'all'])
            run(['yum', 'module', 'enable', '-y',  module_name])
            run(['yum', 'install', '-y', 'bcc'])
            api.current_logger().info("BCC rpms installed.")
