from leapp.actors import Actor
from leapp.models import InstalledRedHatSignedRPM, RpmTransactionTasks
from leapp.libraries.stdlib import api
from leapp.tags import FactsPhaseTag, IPUWorkflowTag

class CheckPython3Conflicts(Actor):
    """
    The actor will remove python36-six and python36-urllib3 to resolve the rpm confliction issue.
    """
    name = 'python3_conflicts_check'
    consumes = (InstalledRedHatSignedRPM, )
    produces = (RpmTransactionTasks, )
    tags = (FactsPhaseTag, IPUWorkflowTag)

    def process(self):
        # Convert installed packages message to list
        packages_list = next(api.consume(InstalledRedHatSignedRPM), InstalledRedHatSignedRPM()).items

        to_remove_pkgs = []
        for pkg in packages_list:
            if pkg.name == 'python36-six' or pkg.name == 'python36-urllib3':
                to_remove_pkgs.append(pkg.name)

        if to_remove_pkgs:
            self.produce(RpmTransactionTasks(to_remove=to_remove_pkgs))
            api.current_logger().info('python3_conflicts_check, remove rpm {0}.'.format(' '.join(to_remove_pkgs)))