from leapp.actors import Actor
from leapp.models import InstalledRedHatSignedRPM, RpmTransactionTasks
from leapp.libraries.stdlib import api
from leapp.tags import FactsPhaseTag, IPUWorkflowTag

def le_than_275_77(version_id, release_id):
    ver_list = str(version_id).split(".")
    std_list = ['2', '7', '5']
    rel_list = str(release_id).split(".")

    try:
        for i in range(len(ver_list)) if len(ver_list) < len(std_list) else range(len(std_list)):
            if int(ver_list[i]) == int(std_list[i]):
                pass
            elif int(ver_list[i]) < int(std_list[i]):
                return True
            else:
                return False

        if len(ver_list) == len(std_list):
            return int(rel_list[0]) <= 77
        else:
            return len(ver_list) < len(std_list)
    except ValueError:
        api.current_logger().error("ValueError during comparing version! {0}-{1} ".format(version_id, release_id))
        return False

class CheckPythonDev(Actor):
    """
    The actor will remove the specific version of python-devel to resolve the rpm dependency issue.
    """
    name = 'python_dev_checks'
    consumes = (InstalledRedHatSignedRPM, )
    produces = (RpmTransactionTasks, )
    tags = (FactsPhaseTag, IPUWorkflowTag)

    def process(self):
        # Convert installed packages message to list
        packages_list = next(api.consume(InstalledRedHatSignedRPM), InstalledRedHatSignedRPM()).items

        python_dev_pkg = None
        for pkg in packages_list:
            if pkg.name == 'python-devel':
                python_dev_pkg = pkg
                break

        if python_dev_pkg:
            release_id = python_dev_pkg.release
            version_id = python_dev_pkg.version
            api.current_logger().debug("python-devel-{0}-{1} is installed.".format(version_id, release_id))


            if le_than_275_77(version_id, release_id):
                api.current_logger().info("Attention: python-devel-{0}-{1} will be removed since version is low.".format(version_id, release_id))
                # todo: add report to inform user
                self.produce(RpmTransactionTasks(to_remove=['python-devel']))
                return

            python3_libs_pkg = None
            for pkg in packages_list:
                if pkg.name == 'python3-libs':
                    python3_libs_pkg = pkg
                    break

            if python3_libs_pkg:
                api.current_logger().info("Attention: python-devel-{0}-{1} will be removed since python3-libs is installed.".format(version_id, release_id))
                # todo: add report to inform user
                self.produce(RpmTransactionTasks(to_remove=['python-devel']))

