import os
import xml.etree.ElementTree as ElementTree

from leapp.actors import Actor
from leapp.libraries.actor import firewalldfactsactor
from leapp.models import FirewalldFacts
from leapp.tags import FactsPhaseTag, IPUWorkflowTag
from leapp import reporting

class FirewalldFactsActor(Actor):
    """
    Provide data about firewalld

    After collecting data, a message with relevant data will be produced.
    """

    name = 'firewalld_facts_actor'
    consumes = ()
    produces = (FirewalldFacts, reporting.Report)
    tags = (FactsPhaseTag, IPUWorkflowTag)

    def process(self):
        facts = FirewalldFacts()
        report_content = [
            reporting.Title('Failed to read firewalld configuration files.'),
            reporting.Summary(''),
            reporting.Severity(reporting.Severity.HIGH),
            reporting.Flags([reporting.Flags.INHIBITOR]),
            reporting.Tags([reporting.Tags.FILESYSTEM]),
            reporting.Remediation(hint=('Check whether XML format is Correct.')),
        ]
        report_summary = None

        try:
            tree = ElementTree.parse('/etc/firewalld/lockdown-whitelist.xml')
            root = tree.getroot()
            facts.firewall_config_command = firewalldfactsactor.getLockdownFirewallConfigCommand(root)
        except IOError:
            pass
        except Exception as e:
            report_summary = reporting.Summary('Got exception when parse {0} files. {1}.'.format('/etc/firewalld/lockdown-whitelist.xml', e))

        try:
            tree = ElementTree.parse('/etc/firewalld/direct.xml')
            root = tree.getroot()
            facts.ebtablesTablesInUse = firewalldfactsactor.getEbtablesTablesInUse(root)
        except IOError:
            pass
        except Exception as e:
            report_summary = reporting.Summary('Got exception when parse {0} files. {1}.'.format('/etc/firewalld/direct.xml', e))

        ipsetTypesInUse = set()
        directory = '/etc/firewalld/ipsets'
        try:
            for filename in os.listdir(directory):
                if not filename.endswith('.xml'):
                    continue
                try:
                    tree = ElementTree.parse(os.path.join(directory, filename))
                    root = tree.getroot()
                    ipsetTypesInUse |= set(firewalldfactsactor.getIpsetTypesInUse(root))
                except Exception as e:
                    report_summary = reporting.Summary('Got exception when parse {0} files. {1}.'.format(directory + '/' + filename, e))
            facts.ipsetTypesInUse = list(ipsetTypesInUse)
        except OSError:
            pass

        if report_summary:
            report_content[1] = report_summary
            reporting.create_report(report_content)

        self.produce(facts)
