import os
import json
import time
import subprocess
from leapp.actors import Actor
from leapp.tags import PostProcessingScriptPhaseTag, IPUWorkflowTag
from leapp.libraries.stdlib import api, CalledProcessError, run
from leapp.models import InstalledRedHatSignedRPM

class PostProcessKernels(Actor):
    """
    process source and target kernel after migration
    """

    name = 'post_process_kernels'
    consumes = (InstalledRedHatSignedRPM,)
    produces = ()
    tags = (PostProcessingScriptPhaseTag, IPUWorkflowTag)

    def process(self):
        # Convert installed packages message to list
        packages_list = next(api.consume(InstalledRedHatSignedRPM), InstalledRedHatSignedRPM()).items

        el7_kernels_nvr = []

        recovery_kernel = ''
        if os.path.isfile('/etc/rear/grub_param.json'):
            with open('/etc/rear/grub_param.json', 'r') as fd:
                try:
                    backup_config = json.load(fd)
                    recovery_kernel = backup_config['vmlinuz_file_path']
                except:
                    recovery_kernel = ''

        for pkg in packages_list:
            if ('kernel' == pkg.name or 'kernel-bek' == pkg.name) and 'el7' in pkg.release:
                if not (pkg.version + '-' + pkg.release in recovery_kernel):
                    el7_kernels_nvr.append(pkg.name + '-' + pkg.version + '-' + pkg.release)
                run(['grubby', '--remove-kernel=/boot/vmlinuz-' + pkg.version + '-' + pkg.release + '.' + pkg.arch], checked=False)

        if el7_kernels_nvr:
            cmd = ['yum', 'remove', '-y']
            cmd.extend(el7_kernels_nvr)
            print('Now remove old kernels rpm, [{}].'.format(' '.join(cmd)))
            r = run(cmd, checked=False)
            print('Result: exit_code: {0}, error: {1}, signal: {2}.'.format(r["exit_code"], r["stderr"], r["signal"]))

        else:
            print('Not Found old kernels need to remove.')

        CONFIGURATION_BASE_PATH = '/etc/bclinux-sysmt/transaction'
        TARGET_KERNEL_CFG_PATH = CONFIGURATION_BASE_PATH + '/target_kernel_nvr'
        target_kernel_cfg_items = []

        if os.path.isfile(TARGET_KERNEL_CFG_PATH):
            try:
                with open(TARGET_KERNEL_CFG_PATH, 'r') as f:
                    for aline in f:
                        entry = aline.strip()
                        if entry and (not entry.startswith('#')):
                            target_kernel_cfg_items.append(entry)
            except IOError as e:
                logger.warning('Failed to open %s to load target kernel configuration. Error: %s', TARGET_KERNEL_CFG_PATH, str(e))

            need_reboot = False
            if target_kernel_cfg_items:
                for cfg_item in target_kernel_cfg_items:
                    if cfg_item.startswith('kernel-') or cfg_item.startswith('kernel-bek-'):
                        print("Now install " + cfg_item)
                        r = run(['yum', 'install', '-y', cfg_item], checked=False)
                        print('Result: exit_code: {0}, error: {1}, signal: {2}.'.format(r["exit_code"], r["stderr"], r["signal"]))
                        if r['exit_code'] == 0:
                            need_reboot = True

                if need_reboot:
                    print("Kernel Install Successfully. Takes effect after reboot!")
            else:
                print('Not Found target kernel configuration, nothing to do.')
                return
