import os
import json
import time
import subprocess
from leapp.actors import Actor
from leapp.tags import PostProcessingScriptPhaseTag, IPUWorkflowTag
from leapp.libraries.stdlib import CalledProcessError, run


class PostProcessingScript(Actor):
    """
    process post-script after migration
    """

    name = 'post_process_script'
    consumes = ()
    produces = ()
    tags = (PostProcessingScriptPhaseTag, IPUWorkflowTag)

    post_script_path = "/var/lib/bclinux-sysmt/customized/scripts/post"

    def process(self):
        if os.path.isdir(self.post_script_path):
            script_dir_list = os.listdir(self.post_script_path)
            script_file_list = []
            for dir_item in script_dir_list:
                dir_item_path = self.post_script_path + '/' + dir_item
                if os.path.isfile(dir_item_path) and os.access(dir_item_path, os.X_OK):
                    script_file_list.append(dir_item)

            if script_file_list:
                script_file_list.sort()
                print('Now execute post-process scripts in order, [{}].'.format(script_file_list))

                script_output_path = '/var/log/bclinux-sysmt/scripts_output'
                subprocess.check_call("mkdir -p {0}".format(script_output_path), shell=True)
                for script_exec in script_file_list:
                    script_exec_path = self.post_script_path + '/' + script_exec
                    output_file = script_output_path + "/post_proc_" + time.strftime("%Y-%m-%d-%H-%M-%S", time.localtime()) + ".out"
                    r = run([script_exec_path])
                    print('Script: {0}, exit_code: {1}, pid: {2}, signal: {3}.'.format(script_exec, r["exit_code"], r["pid"], r["signal"]))

                    with open(output_file, 'w') as out_fd:
                        out_fd.write("Script stderr:\n")
                        out_fd.write(r["stderr"])
                        out_fd.write('\n')
                        out_fd.write("Script stdout:\n")
                        out_fd.write(r["stdout"])
        else:
            print('Not exist post process scripts in {}, nothing to do.'.format(self.post_script_path))
            return
