from re import compile as regexp
import os

from leapp.libraries.stdlib import api, run

_current_boot_matcher = regexp(r'BootCurrent: (?P<boot_current>([0-9A-F]*))')
_next_boot_matcher = regexp(r'BootNext: (?P<boot_next>([0-9A-F]*))')


def get_current_boot_match(string):
    match = _current_boot_matcher.match(string)
    if not match:
        return None
    captured_groups = match.groupdict()
    return captured_groups['boot_current']


def get_next_boot_match(string):
    match = _next_boot_matcher.match(string)
    if not match:
        return None
    captured_groups = match.groupdict()
    return captured_groups['boot_next']


def maybe_emit_updated_boot_entry(is_final):
    if not os.path.exists('/sbin/efibootmgr'):
        return

    efi_info = run(['/sbin/efibootmgr'], checked=False, split=True)
    if efi_info['exit_code'] != 0:
        # Not an efi system
        return

    current_boot, next_boot = None, None
    for line in efi_info['stdout']:
        current_match = get_current_boot_match(line)
        if current_match:
            current_boot = current_match

        next_match = get_next_boot_match(line)
        if next_match:
            next_boot = next_match

    if current_boot and not next_boot:
        # We set BootNext to CurrentBoot only if BootNext wasn't previously set
        run(['/sbin/efibootmgr', '-n', current_boot])

    if is_final is True:
        grub_cfg_path = os.readlink('/etc/grub2-efi.cfg').strip('.')
        if not os.path.isfile(grub_cfg_path):
            api.current_logger().info("{} Not Exist! Now fix it.".format(grub_cfg_path))
            run(['mkdir', '-p', os.path.dirname(grub_cfg_path)])

            rst = run(['find', '/boot/efi', '-name', 'grub.cfg', '-mmin', '-30'])
            target_grub_cfg = ""
            if rst['stdout']:
                api.current_logger().info("Found {}".format(rst['stdout']))
                target_grub_cfg = rst['stdout'].split('\n')[0]

            if target_grub_cfg:
                run(['cp', target_grub_cfg, grub_cfg_path])
            else:
                if os.path.isfile('/boot/efi/EFI/centos/grub.cfg'):
                    run(['cp', '/boot/efi/EFI/centos/grub.cfg', grub_cfg_path])
                elif os.path.isfile('/boot/efi/EFI/redhat/grub.cfg'):
                    run(['cp', '/boot/efi/EFI/redhat/grub.cfg', grub_cfg_path])
                else:
                    run(['cp', '/boot/efi/EFI/bclinux/grub.cfg', grub_cfg_path])

